Class: Pgtk::Pool::IterableQueue
- Inherits:
-
Object
- Object
- Pgtk::Pool::IterableQueue
- Defined in:
- lib/pgtk/pool.rb
Overview
Thread-safe queue implementation that supports iteration. Unlike Ruby’s Queue class, this implementation allows safe iteration over all elements while maintaining thread safety for concurrent access.
This class is used internally by Pool to store database connections and provide the ability to iterate over them for inspection purposes.
The queue is bounded by size. When an item is taken out, it remains in the internal array but is marked as “taken”. When returned, it’s placed back in its original slot and marked as available.
Instance Method Summary collapse
-
#initialize(size, timeout) ⇒ IterableQueue
constructor
A new instance of IterableQueue.
- #map ⇒ Object
- #pop ⇒ Object
- #push(item) ⇒ Object
- #size ⇒ Object
Constructor Details
#initialize(size, timeout) ⇒ IterableQueue
Returns a new instance of IterableQueue.
227 228 229 230 231 232 233 234 235 |
# File 'lib/pgtk/pool.rb', line 227 def initialize(size, timeout) @size = size @timeout = timeout @items = [] @taken = [] @free = [] @mutex = Mutex.new @condition = ConditionVariable.new end |
Instance Method Details
#map ⇒ Object
277 278 279 280 281 |
# File 'lib/pgtk/pool.rb', line 277 def map(&) @mutex.synchronize do @items.map(&) end end |
#pop ⇒ Object
257 258 259 260 261 262 263 264 265 266 267 268 269 |
# File 'lib/pgtk/pool.rb', line 257 def pop @mutex.synchronize do deadline = Process.clock_gettime(Process::CLOCK_MONOTONIC) + @timeout while @free.empty? remaining = deadline - Process.clock_gettime(Process::CLOCK_MONOTONIC) raise(Busy, "No free connection appeared in the pool after #{@timeout}s of waiting") if remaining <= 0 @condition.wait(@mutex, remaining) end index = @free.shift @taken[index] = true @items[index] end end |
#push(item) ⇒ Object
237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 |
# File 'lib/pgtk/pool.rb', line 237 def push(item) @mutex.synchronize do if @items.size < @size @items << item @taken << false @free << (@items.size - 1) else index = @items.index(item) if index.nil? index = @taken.index(true) raise(StandardError, 'No taken slot found') if index.nil? @items[index] = item end @taken[index] = false @free << index end @condition.signal end end |
#size ⇒ Object
271 272 273 274 275 |
# File 'lib/pgtk/pool.rb', line 271 def size @mutex.synchronize do @items.size end end |